{% import "snippets.jinja2" as snip %}
{{ snip.pair_entries(modality, entry_square, entry_circle, circle_first=circle_first|default(false)) }}

{% if differentiate %}
Your task is to list concise, mutually exclusive terms or short phrases that best explain how entry "circle" differs from entry "square".
These terms should be specific and reflect clear differences observable between these particular entries.
There can be any number of terms from none up; do not invent terms if there is no meaningful difference.
Compile all relevant differences; be thorough.

When listing differences, explicitly note whether "square" or "circle" shows more or less of the attribute. For example, say "circle has more X than square" or "square lacks Y whereas circle has some".
Each extracted term should be a directional statement (e.g. "square exhibits strong communication amongst colleagues, circle does not", "circle uses lots of first person perspective whereas square rarely does", "square involves more chronic health issues than circle", "complications from type 2 diabetes seen in square but not circle", "circle has no direct quotes from evidence, square has some", etc) rather than an "X vs Y" phrase.
Strong preference for descriptive, highly specific, and directional terms-avoid vague and broad terms, and terms where it is unclear what it means for an entry to have more or less of the term.
Again: all about differences. An attribute shown in one entry but not the other is a difference. An attribute shown to a greater extent in one entry than the other is a difference.

Shortened example output:
{
    "circle mentions renewable energy more than square": "Circle discusses wind and solar incentives extensively (stating 'A' and 'B'), whereas square only briefly references green jobs (focusing instead on 'C').",
    "square uses historic evidence and anecdotes while circle does not": "Square highlights historical evidence and anecdotes in its argument (citing 'D', 'E'), whereas circle invokes no details from history.",
    ...
}
Stick closely to this "square focuses more on ... than circle", "circle advocates for ... more than square", "square uses ... while circle does not" general style in your term names.
{% else %}
Your task is to list concise, mutually exclusive terms or short phrases that best explain how entry "circle" is similar to entry "square".
These terms should be specific and reflect clear similarities observable between these particular entries.
There can be any number of terms from none up; do not invent terms if there is no meaningful similarity.
Compile all relevant similarities; be thorough.

Shortened example output:
{
    "focus on renewable energy": "Both entries discuss wind and solar incentives extensively. Entry circle says 'A' and 'B', and entry square shows a similar focus, stating 'C' and 'D'.",
    "use of historic evidence and anecdotes": "Both passages highlight historical evidence and anecdotes in their arguments. Entry square cites 'E'; entry circle invokes 'F' and 'G'.",
    ...
}
{% endif %}

Don't be vague or generic. Be precise and parsimonious.
Each term must have a brief justification, directly quoting/using evidence from the entries where relevant.
No terms, one term, three, six, many terms may apply-don't skip any applicable terms but output zero/few if that is the correct choice.
Avoid trivial and pedantic terms; only substantive, meaningful terms-zero/fewer terms better than pointless trivialities.
Ensure your terms are mutually exclusive; each should be a distinct idea and must not overlap with other terms.
Include both obvious and subtle but relevant observations-think broadly and approach the comparison from different angles.
Employ creative thinking to assess from many different angles so that you notice non-obvious (but still relevant) points of comparison too.
ABSOLUTELY no underscores/hyphens in your term names-just natural, parsimonious language, like the examples.

Output JSON only, mapping each term to a 1-3 sentence justification:
{
    "<insert term name here>": "<insert direct quotes/evidence and reasoning as justification (1-3 evidence dense sentences)>",
    "<term>": "...",
    ...
}

{% if additional_instructions %}
The user has provided the following additional instructions, clarifications, or labelled examples (if these provided, rely heavily on them to calibrate yourself).
If they conflict with the other instructions, these user instructions take precedence.
If they specify a specific scope to confine your terms to, it is essential you comply with that scope, focus closely on it, and not output any terms outside of it.

BEGIN ADDITIONAL INSTRUCTIONS/CLARIFICATIONS
{{ additional_instructions }}
END ADDITIONAL INSTRUCTIONS/CLARIFICATIONS
{% endif %}
