Consider the following instructions on the sort of entities we wish to create:

BEGIN ENTITY INSTRUCTIONS
{{ instructions }}
END ENTITY INSTRUCTIONS

Your task is to generate {{ entities_per_generation }} such entities in accordance with the above instructions.
It is crucial that each entity is distinct from the others; ensure mutual exclusivity and accurate/representative coverage of the whole domain of entities specified in the instructions.
Do not repeat yourself; each entity in your output should be its own unique creation from each other and each must tread new ground and go in a novel direction.
If the instructions specify a distribution, ensure the overall set of entities represent that distribution accurately (NOT overrepresenting minorities, but accurately capturing the true statistical reality of said distribution).

For example, if asked to initialize personas that represent the US population, you would use your knowledge to ensure the overall set of entities matches known statistics, such as the fraction of entities of each race or state or educational background or job or personality traits.
In this case, carefully consider existing entities and your knowledge of the true statistical distributions; if a certain characteristic appears overrepresented (according to STATISTICS not perception), downweight it in your output as to rebalance the overall set (e.g. too many women, output fewer women to reach true distribution; too few Christians, output extra).
If a certain demographic is only 1% of the population, it should rarely be included in output at all and avoided, to ensure its prevalence in the total set is accurate.
Consider fraction of your {{ entities_per_generation }} entities that should reasonably be devoted to any group (i.e. a 7% demographic should only get that percent of slots, and less if previously overrepresented); rare demographics should get none or very few slots, based on their population frequency.

If the instructions instead emphasize true creativity and novelty, you must put extra effort into each entity going in a genuinely different direction than the others, with some going in truly unusual and hypercreative directions to ensure mutual exclusivity and radical novelty.
For example, if asked to generate novel areas for scientific research, each idea should be specific but going in a clearly novel direction from the others.
Mutual exclusivity and truly different approaches are key; each should chart its own unique path.
If some topic is overrepresented in the existing entities, avoid that area and go somewhere completely new.

{% if existing_entities %}
The following are a sample of entities you have already generated.
In addition to making each element of your generated entity list unique from one another, you must ensure they are unique from these too.
Carefully read these existing entities to ensure your don't tread the same ground.
BEGIN EXISTING ENTITIES
{{ existing_entities }}
END EXISTING ENTITIES
Again: your new entities must not repeat what has already been done.
Your set of entities is ALL of them (existing + new generations); consider the entire set as one.
{% endif %}

Output in the following JSON format:
BEGIN OUTPUT FORMAT
{
    "entity 1": "<insert unique entity here>",
    "entity 2": "<insert entity here, unique from others>",
    ...,
    "entity n": "<insert unique entity here>"
}
END OUTPUT FORMAT

Ensure you output exactly {{ entities_per_generation }} unique entities.
